@@ -8,11 +8,12 @@ Google Spreadsheets client library.
 
 """
 
-__version__ = '0.0.12'
+__version__ = '0.0.13'
 __author__ = 'Anton Burnashev'
 
 from .client import Client, login
 from .models import Spreadsheet, Worksheet, Cell
 from .exceptions import (GSpreadException, AuthenticationError,
                          SpreadsheetNotFound, NoValidUrlKeyFound,
-                         IncorrectCellLabel, WorksheetNotFound)
+                         IncorrectCellLabel, WorksheetNotFound,
+                         UpdateCellError, RequestError)
