@@ -14,12 +14,14 @@ from xml.etree import ElementTree
 
 from . import __version__
 from .ns import _ns
-from .httpsession import HTTPSession
+from .httpsession import HTTPSession, HTTPError
 from .models import Spreadsheet
 from .urls import construct_url
 from .utils import finditem
 from .exceptions import (AuthenticationError, SpreadsheetNotFound,
-                         NoValidUrlKeyFound)
+                         NoValidUrlKeyFound, UpdateCellError,
+                         RequestError)
+
 
 AUTH_SERVER = 'https://www.google.com'
 SPREADSHEETS_SERVER = 'spreadsheets.google.com'
@@ -76,22 +78,24 @@ class Client(object):
 
         url = AUTH_SERVER + '/accounts/ClientLogin'
 
-        r = self.session.post(url, data)
-        content = r.read().decode()
-
-        if r.code == 200:
+        try:
+            r = self.session.post(url, data)
+            content = r.read().decode()
             token = self._get_auth_token(content)
             auth_header = "GoogleLogin auth=%s" % token
             self.session.add_header('Authorization', auth_header)
 
-        elif r.code == 403:
-            if content.strip() == 'Error=BadAuthentication':
-                raise AuthenticationError("Incorrect username or password")
+        except HTTPError as ex:
+            if ex.code == 403:
+                content = ex.read().decode()
+                if content.strip() == 'Error=BadAuthentication':
+                    raise AuthenticationError("Incorrect username or password")
+                else:
+                    raise AuthenticationError("Unable to authenticate. %s code" % ex.code)
+
             else:
-                raise AuthenticationError("Unable to authenticate. %s code" % r.code)
+                raise AuthenticationError("Unable to authenticate. %s code" % ex.code)
 
-        else:
-            raise AuthenticationError("Unable to authenticate. %s code" % r.code)
 
     def open(self, title):
         """Opens a spreadsheet, returning a :class:`~gspread.Spreadsheet` instance.
@@ -208,6 +212,10 @@ class Client(object):
         r = self.session.get(url)
         return ElementTree.fromstring(r.read())
 
+    def get_feed(self, url):
+        r = self.session.get(url)
+        return ElementTree.fromstring(r.read())
+
     def get_cells_cell_id_feed(self, worksheet, cell_id,
                        visibility='private', projection='full'):
         url = construct_url('cells_cell_id', worksheet, cell_id=cell_id,
@@ -216,10 +224,30 @@ class Client(object):
         r = self.session.get(url)
         return ElementTree.fromstring(r.read())
 
-    def put_cell(self, url, data):
+    def put_feed(self, url, data):
         headers = {'Content-Type': 'application/atom+xml'}
         data = self._add_xml_header(data)
-        r = self.session.put(url, data, headers=headers)
+
+        try:
+            r = self.session.put(url, data, headers=headers)
+        except HTTPError as ex:
+            if ex.code == 403:
+                message = ex.read().decode()
+                raise UpdateCellError(message)
+            else:
+                raise ex
+
+        return ElementTree.fromstring(r.read())
+
+    def post_feed(self, url, data):
+        headers = {'Content-Type': 'application/atom+xml'}
+        data = self._add_xml_header(data)
+
+        try:
+            r = self.session.post(url, data, headers=headers)
+        except HTTPError as ex:
+            message = ex.read().decode()
+            raise RequestError(message)
 
         return ElementTree.fromstring(r.read())
 
