@@ -8,15 +8,36 @@ This module contains common spreadsheets' models
 
 """
 import re
+from collections import defaultdict
+from itertools import chain
 
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element, SubElement
 
-from .ns import _ns, _ns1, ATOM_NS, BATCH_NS, GS_NS
+from .ns import _ns, _ns1, ATOM_NS, BATCH_NS, SPREADSHEET_NS
 from .urls import construct_url
-from .utils import finditem
+from .utils import finditem, numericise_all
+
+from .exceptions import IncorrectCellLabel, WorksheetNotFound, CellNotFound
+
+
+try:
+    unicode
+except NameError:
+    basestring = unicode = str
+
+
+# Patch ElementTree._escape_attrib
+_elementtree_escape_attrib = ElementTree._escape_attrib
+def _escape_attrib(text, encoding=None, replace=None):
+    text = _elementtree_escape_attrib(text, encoding)
+    entities = {'\n': '&#10;', '\r': '&#13;', '\t':'&#9;'}
+    for key, value in entities.items():
+        text = text.replace(key, value)
+    return text
+
+ElementTree._escape_attrib = _escape_attrib
 
-from .exceptions import IncorrectCellLabel, WorksheetNotFound
 
 class Spreadsheet(object):
     """A class for a spreadsheet object.
@@ -36,6 +57,30 @@ class Spreadsheet(object):
         for elem in feed.findall(_ns('entry')):
             self._sheet_list.append(Worksheet(self, elem))
 
+    def add_worksheet(self, title, rows, cols):
+        """Adds a new worksheet to a spreadsheet.
+
+        :param title: A title of a new worksheet.
+        :param rows: Number of rows.
+        :param cols: Number of columns.
+
+        Returns a newly created :class:`worksheets <Worksheet>`.
+        """
+        feed = Element('entry', {'xmlns': ATOM_NS,
+                                'xmlns:gs': SPREADSHEET_NS})
+
+        SubElement(feed, 'title').text = title
+        SubElement(feed, 'gs:rowCount').text = str(rows)
+        SubElement(feed, 'gs:colCount').text = str(cols)
+
+        url = construct_url('worksheets', self)
+        elem = self.client.post_feed(url, ElementTree.tostring(feed))
+
+        worksheet = Worksheet(self, elem)
+        self._sheet_list.append(worksheet)
+
+        return worksheet
+
     def worksheets(self):
         """Returns a list of all :class:`worksheets <Worksheet>` in a spreadsheet.
 
@@ -80,10 +125,14 @@ class Spreadsheet(object):
         >>> sht = client.open('My fancy spreadsheet')
         >>> worksheet = sht.get_worksheet(0)
 
+        Returns `None` if the worksheet is not found.
         """
         if not self._sheet_list:
             self._fetch_sheets()
-        return self._sheet_list[index]
+        try:
+            return self._sheet_list[index]
+        except IndexError:
+            return None
 
     @property
     def sheet1(self):
@@ -139,14 +188,15 @@ class Worksheet(object):
     def _cell_addr(self, row, col):
         return 'R%sC%s' % (row, col)
 
+    def _get_link(self, link_type, feed):
+        return finditem(lambda x: x.get('rel') == link_type,
+                feed.findall(_ns('link')))
+
     def _fetch_cells(self):
         feed = self.client.get_cells_feed(self)
-        cells_list = []
-        for elem in feed.findall(_ns('entry')):
-            cells_list.append(Cell(self, elem))
-
-        return cells_list
+        return [Cell(self, elem) for elem in feed.findall(_ns('entry'))]
 
+    _MAGIC_NUMBER = 64
     _cell_addr_re = re.compile(r'([A-Za-z]+)(\d+)')
     def get_int_addr(self, label):
         """Translates cell's label address to a tuple of integers.
@@ -157,24 +207,57 @@ class Worksheet(object):
                       Letter case is ignored.
 
         Example:
+
         >>> wks.get_int_addr('A1')
         (1, 1)
 
         """
-        magic_number = 96
         m = self._cell_addr_re.match(label)
         if m:
-            column_label = m.group(1).lower()
+            column_label = m.group(1).upper()
             row = int(m.group(2))
 
             col = 0
             for i, c in enumerate(reversed(column_label)):
-                col += (ord(c) - magic_number) * (26 ** i)
+                col += (ord(c) - self._MAGIC_NUMBER) * (26 ** i)
         else:
             raise IncorrectCellLabel(label)
 
         return (row, col)
 
+    def get_addr_int(self, row, col):
+        """Translates cell's tuple of integers to a cell label.
+
+        The result is a string containing the cell's coordinates in label form.
+
+        :param row: The row of the cell to be converted.
+                    Rows start at index 1.
+
+        :param col: The column of the cell to be converted.
+                    Columns start at index 1.
+
+        Example:
+
+        >>> wks.get_addr_int(1, 1)
+        A1
+
+        """
+        row = int(row)
+        col = int(col)
+
+        if row < 1 or col < 1:
+            raise IncorrectCellLabel('(%s, %s)' % (row, col))
+
+        div = col
+        column_label = str()
+
+        while div:
+            (div, mod) = divmod(div, 26)
+            column_label = chr(mod+self._MAGIC_NUMBER) + column_label
+
+        label = '%s%s' % (column_label, row)
+        return label
+
     def acell(self, label):
         """Returns an instance of a :class:`Cell`.
 
@@ -218,25 +301,56 @@ class Worksheet(object):
         return [Cell(self, elem) for elem in feed.findall(_ns('entry'))]
 
     def get_all_values(self):
-        """Returns a list of lists containing all cells' values."""
+        """Returns a list of lists containing all cells' values as strings."""
         cells = self._fetch_cells()
 
-        rows = {}
+        # defaultdicts fill in gaps for empty rows/cells not returned by gdocs
+        rows = defaultdict(lambda: defaultdict(str))
         for cell in cells:
-            rows.setdefault(int(cell.row), []).append(cell)
+            row = rows.setdefault(int(cell.row), defaultdict(str))
+            row[cell.col] = cell.value
 
-        rows_list = []
-        for r in sorted(rows.keys()):
-            rows_list.append(rows[r])
+        # we return a whole rectangular region worth of cells, including empties
+        all_row_keys = chain.from_iterable(row.keys() for row in rows.values())
+        rect_cols = range(1, max(all_row_keys)+1)
+        rect_rows = range(1, max(rows.keys())+1)
 
-        simple_rows_list = []
-        for r in rows_list:
-            simple_row = []
-            for c in r:
-                simple_row.append(c.value)
-            simple_rows_list.append(simple_row)
+        return [[rows[i][j] for j in rect_cols] for i in rect_rows]
 
-        return simple_rows_list
+    def get_all_records(self, empty2zero=False):
+        """Returns a list of dictionaries, all of them having:
+            - the contents of the spreadsheet's first row of cells as keys,
+            And each of these dictionaries holding
+            - the contents of subsequent rows of cells as values.
+
+
+        Cell values are numericised (strings that can be read as ints
+        or floats are converted).
+
+        :param empty2zero: determines whether empty cells are converted to zeros."""
+
+        data = self.get_all_values()
+        keys = data[0]
+        values = [numericise_all(row, empty2zero) for row in data[1:]]
+
+        return [dict(zip(keys, row)) for row in values]
+
+
+    def _list_values(self, index, cell_tuple, position):
+        cells_list = self._fetch_cells()
+        cells = dict(map(cell_tuple, filter(position, cells_list)))
+
+        try:
+            last_index = max(cells.keys())
+        except ValueError:
+            return []
+
+        vals = []
+        for i in range(1, last_index + 1):
+            c = cells.get(i)
+            vals.append(c.value if c else None)
+
+        return vals
 
     def row_values(self, row):
         """Returns a list of all values in a `row`.
@@ -244,20 +358,9 @@ class Worksheet(object):
         Empty cells in this list will be rendered as :const:`None`.
 
         """
-        cells_list = self._fetch_cells()
-
-        cells = {}
-        for cell in cells_list:
-            if int(cell.row) == row:
-                cells[int(cell.col)] = cell
-
-        last_index = max(cells.keys())
-        vals = []
-        for i in range(1, last_index + 1):
-            c = cells.get(i)
-            vals.append(c.value if c else None)
-
-        return vals
+        return self._list_values(row,
+                                 lambda cell: (cell.col, cell),
+                                 lambda cell: cell.row == row)
 
     def col_values(self, col):
         """Returns a list of all values in column `col`.
@@ -265,20 +368,9 @@ class Worksheet(object):
         Empty cells in this list will be rendered as :const:`None`.
 
         """
-        cells_list = self._fetch_cells()
-
-        cells = {}
-        for cell in cells_list:
-            if int(cell.col) == col:
-                cells[int(cell.row)] = cell
-
-        last_index = max(cells.keys())
-        vals = []
-        for i in range(1, last_index + 1):
-            c = cells.get(i)
-            vals.append(c.value if c else None)
-
-        return vals
+        return self._list_values(col,
+                                 lambda cell: (cell.row, cell),
+                                 lambda cell: cell.col == col)
 
     def update_acell(self, label, val):
         """Sets the new value to a cell.
@@ -306,17 +398,15 @@ class Worksheet(object):
         feed = self.client.get_cells_cell_id_feed(self,
                                                   self._cell_addr(row, col))
         cell_elem = feed.find(_ns1('cell'))
-        cell_elem.set('inputValue', val)
-        edit_link = finditem(lambda x: x.get('rel') == 'edit',
-                feed.findall(_ns('link')))
-        uri = edit_link.get('href')
+        cell_elem.set('inputValue', unicode(val))
+        uri = self._get_link('edit', feed).get('href')
 
-        self.client.put_cell(uri, ElementTree.tostring(feed))
+        self.client.put_feed(uri, ElementTree.tostring(feed))
 
     def _create_update_feed(self, cell_list):
         feed = Element('feed', {'xmlns': ATOM_NS,
                                 'xmlns:batch': BATCH_NS,
-                                'xmlns:gs': GS_NS})
+                                'xmlns:gs': SPREADSHEET_NS})
 
         id_elem = SubElement(feed, 'id')
 
@@ -338,7 +428,7 @@ class Worksheet(object):
 
             SubElement(entry, 'gs:cell', {'row': str(cell.row),
                                           'col': str(cell.col),
-                                          'inputValue': str(cell.value)})
+                                          'inputValue': unicode(cell.value)})
         return feed
 
     def update_cells(self, cell_list):
@@ -350,6 +440,91 @@ class Worksheet(object):
         feed = self._create_update_feed(cell_list)
         self.client.post_cells(self, ElementTree.tostring(feed))
 
+    def resize(self, rows=None, cols=None):
+        """Resizes the worksheet.
+
+        :param rows: New rows number.
+        :param cols: New columns number.
+        """
+        if rows is None and cols is None:
+            raise TypeError("Either 'rows' or 'cols' should be specified.")
+
+        self_uri = self._get_link('self', self._element).get('href')
+        feed = self.client.get_feed(self_uri)
+        uri = self._get_link('edit', feed).get('href')
+
+        if rows:
+            elem = feed.find(_ns1('rowCount'))
+            elem.text = str(rows)
+
+        if cols:
+            elem = feed.find(_ns1('colCount'))
+            elem.text = str(cols)
+
+        # Send request and store result
+        self._element = self.client.put_feed(uri, ElementTree.tostring(feed))
+
+    def add_rows(self, rows):
+        """Adds rows to worksheet.
+
+        :param rows: Rows number to add.
+        """
+        self.resize(rows=self.row_count + rows)
+
+    def add_cols(self, cols):
+        """Adds colums to worksheet.
+
+        :param cols: Columns number to add.
+        """
+        self.resize(cols=self.col_count + cols)
+
+    def append_row(self, values):
+        """"Adds a row to the worksheet and populates it with values.
+        Widens the worksheet if there are more values than columns.
+
+        :param values: List of values for the new row.
+        """
+        self.add_rows(1)
+        new_row = self.row_count
+        data_width = len(values)
+        if self.col_count < data_width:
+            self.resize(cols=data_width)
+
+        cell_list = []
+        for i, value in enumerate(values, start=1):
+            cell = self.cell(new_row, i)
+            cell.value = value
+            cell_list.append(cell)
+
+        self.update_cells(cell_list)
+
+    def _finder(self, func, query):
+        cells = self._fetch_cells()
+
+        if isinstance(query, basestring):
+            match = lambda x: x.value == query
+        else:
+            match = lambda x: query.search(x.value)
+
+        return func(match, cells)
+
+    def find(self, query):
+        """Finds first cell matching query.
+
+        :param query: A text string or compiled regular expression.
+        """
+        try:
+            return self._finder(finditem, query)
+        except StopIteration:
+            raise CellNotFound(query)
+
+    def findall(self, query):
+        """Finds all cells matching query.
+
+        :param query: A text string or compiled regular expression.
+        """
+        return self._finder(filter, query)
+
 
 class Cell(object):
     """An instance of this class represents a single cell in a :class:`worksheet <Worksheet>`.
@@ -375,7 +550,7 @@ class Cell(object):
         return self._col
 
     def __repr__(self):
-        return '<%s R%sC%s "%s">' % (self.__class__.__name__,
+        return '<%s R%sC%s %s>' % (self.__class__.__name__,
                                      self.row,
                                      self.col,
-                                     self.value)
+                                     repr(self.value))
